#!/usr/bin/env python

"""
Directory input context.

Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.input.common import Input
from moinformat.utils.directory import Directory
from os.path import sep

class DirectoryInput(Input):

    "A directory output context."

    name = "directory"

    def __init__(self, parameters=None):

        "Initialise the context with the given 'parameters'."

        if not parameters or not parameters.has_key("filename"):
            raise ValueError, parameters

        Input.__init__(self, parameters)
        self.dir = Directory(parameters["filename"])

        # Support an encoding of the level separator for the filesystem.
        # Where it is the same as the directory separator, documents are stored
        # using nested directories, not as a flat list.

        self.level_sep = parameters and parameters.get("separator") or sep

    def all(self):

        "Return all pages in the context."

        # Ignore dotfiles.

        return map(self.to_pagename, self.dir.select_files("[!.]*"))

    # Page characteristics.

    def subpage_filenames(self, pagename):

        "Return the subpage filenames of 'pagename'."

        pattern = self.to_filename("%s/*" % pagename)
        return self.dir.select_files(pattern)

    def subpages(self, pagename):

        "Return the subpages of 'pagename'."

        return map(self.to_pagename, self.subpage_filenames(pagename))

    # Page access methods.

    def readfile(self, filename, encoding=None):

        """
        Return the contents of the file having the given 'filename' and optional
        'encoding'.
        """

        return self.readpath(self.dir.get_filename(filename), encoding)

    # NOTE: Translation methods should encode filenames appropriately.

    def to_filename(self, pagename):

        "Return the filename corresponding to 'pagename'."

        if sep == self.level_sep:
            return pagename
        else:
            return self.level_sep.join(pagename.split("/"))

    def to_pagename(self, filename):

        "Return the pagename corresponding to 'filename'."

        if sep == self.level_sep:
            return filename
        else:
            return "/".join(filename.split(self.level_sep))

input = DirectoryInput

# vim: tabstop=4 expandtab shiftwidth=4
